import pigpio

from compLib.ADC import ADC
from compLib.LogstashLogging import Logging

TOP_LEFT_CHANNEL = 0
TOP_RIGHT_CHANNEL = 1

TOP_IR_MIN_VOLTAGE = 0.0
TOP_IR_MAX_VOLTAGE = 3.3

BOTTOM_LEFT_PIN = 14
BOTTOM_MIDDLE_PIN = 15
BOTTOM_RIGHT_PIN = 23

adc = ADC()

# GPIO.setmode(GPIO.BCM)

GPIO = pigpio.pi()

GPIO.set_mode(BOTTOM_LEFT_PIN, pigpio.INPUT)
GPIO.set_mode(BOTTOM_MIDDLE_PIN, pigpio.INPUT)
GPIO.set_mode(BOTTOM_RIGHT_PIN, pigpio.INPUT)

states = {
        str(BOTTOM_LEFT_PIN): 0,
        str(BOTTOM_MIDDLE_PIN): 0,
        str(BOTTOM_RIGHT_PIN): 0}

def gpio_callback(gpio, level, tick):
    states[str(gpio)] = level
#    print(gpio, level, tick)

GPIO.callback(BOTTOM_LEFT_PIN, pigpio.EITHER_EDGE, gpio_callback)
GPIO.callback(BOTTOM_MIDDLE_PIN, pigpio.EITHER_EDGE, gpio_callback)
GPIO.callback(BOTTOM_RIGHT_PIN, pigpio.EITHER_EDGE, gpio_callback)

class IRSensor(object):
    """Access the different IR Sensors at top / bottom of the robot
    """

    @staticmethod
    def top_left_percent() -> int:
        """Get top left infrared sensor percentage

        :return: Percentage between 0 and 100
        :rtype: int
        """
        voltage = adc.read(TOP_LEFT_CHANNEL)
        return int((voltage - TOP_IR_MIN_VOLTAGE) / TOP_IR_MAX_VOLTAGE * 100)

    @staticmethod
    def top_right_percent() -> int:
        """Get top right infrared sensor percentage

        :return: Percentage between 0 and 100
        :rtype: int
        """
        voltage = adc.read(TOP_RIGHT_CHANNEL)
        return int((voltage - TOP_IR_MIN_VOLTAGE) / TOP_IR_MAX_VOLTAGE * 100)

    @staticmethod
    def bottom_left() -> bool:
        """Get bottom left infrared sensor status

        :return: True, if sensor detects IR signals
        :rtype: bool
        """
        return states[str(BOTTOM_LEFT_PIN)]
        # return GPIO.read(BOTTOM_LEFT_PIN)

    @staticmethod
    def bottom_middle() -> bool:
        """Get bottom middle infrared sensor status

        :return: True, if sensor detects IR signals
        :rtype: bool
        """
        return states[str(BOTTOM_MIDDLE_PIN)]
        # return GPIO.read(BOTTOM_MIDDLE_PIN)

    @staticmethod
    def bottom_right() -> bool:
        """Get bottom right infrared sensor status

        :return: True, if sensor detects IR signals
        :rtype: bool
        """
        return states[str(BOTTOM_RIGHT_PIN)]
        # return GPIO.read(BOTTOM_RIGHT_PIN)
